scrPath = File::dirname(__FILE__.gsub(/\\/, '/')) + '/'
require scrPath + 'ddd.rb'
#require 'pp'

module OED

  DocDBDir = 'Collects/main/'
  DocPartsDir = 'parts/'
  DocPddDir = 'pdd/'
#  DocPddFname = '00000059.pdd'   # v3.0x
  DocPddFname = '00000060.pdd'   # v3.1

class DocDB

  include Enumerable

  def initialize(root)
    @root = root.sub(/[\/\\]$/, '') + '/'
    makeIDtable(@root)
    @curfname = ''
  end

  def get(id)
    return nil if (id < 1) or (id > @size)

    partsfname = ''
    @idTable.each do |fname, startID, count|
      if id >= startID
        partsfname = fname
        break
      end
    end

    if @curfname != partsfname
      @partsf.close unless @curfname.empty?
      @partsf = DDDFile::open(partsfname + '.ddd', 'rb')
#p @partsf.path
      @curfname = partsfname
      @partsf.init(@root)
    end

    return @partsf.get(id)
  end

  def size
    @size
  end

  def each
    1.upto(@size) do |n|
      yield @partsf.get(n)
    end
  end

  def close
    @partsf.close unless @curfname.empty?
  end

  private

  def makeIDtable(root)
    @idTable = []
    path = root + DocDBDir + DocPddDir
    infname = path + DocPddFname
    inf = open(infname, 'rb')

    inf.read(64)

    @size = 0
    while true do
      type, = inf.read(4).unpack('Z4')
#p type
      break if type.empty?
      size, x, pos, = inf.read(56).unpack('VVvvV*')
      if type == 'pdv'
        inf.seek(pos * 1024)
        inf.read(32)
        while true do
          fname, = inf.read(18).unpack('Z*')
          break if fname.empty?
          startID, count, = inf.read(17).unpack('VVVVC')
          @idTable.push([path + fname, startID, count])
          @size += count
        end
        break
      end
    end
   @idTable.reverse!
#p @idTable, @size
#pp @idTable # [@idTable.size - 1]
  end

end

end ##### module ################################################

if __FILE__ == $0

  include OED

  doc = DocDB::new('H:/OED')

  [411873, 419967, 425956, 457583, 524463, 547226, 598776].each do |n|
    puts '%08X (%d)' % [n, n]
    data = doc.get(n)
    data.each_byte do |b|
      printf('%02X ', b)
    end
    puts ''
  end

  doc.close

end

__END__
[["H:/OED/Collects/main/pdd/../parts/00000060", 3773319, 64024],
 ["H:/OED/Collects/main/pdd/../parts/00000059", 3709290, 64029],
 ["H:/OED/Collects/main/pdd/../parts/00000058", 3645272, 64018],
 ["H:/OED/Collects/main/pdd/../parts/00000057", 3580954, 64318],
 ["H:/OED/Collects/main/pdd/../parts/00000056", 3516952, 64002],
 ["H:/OED/Collects/main/pdd/../parts/00000055", 3452952, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000054", 3388897, 64055],
 ["H:/OED/Collects/main/pdd/../parts/00000053", 3324893, 64004],
 ["H:/OED/Collects/main/pdd/../parts/00000052", 3260872, 64021],
 ["H:/OED/Collects/main/pdd/../parts/00000051", 3196700, 64172],
 ["H:/OED/Collects/main/pdd/../parts/00000050", 3132692, 64008],
 ["H:/OED/Collects/main/pdd/../parts/00000049", 3068088, 64604],
 ["H:/OED/Collects/main/pdd/../parts/00000048", 3004078, 64010],
 ["H:/OED/Collects/main/pdd/../parts/00000047", 2940073, 64005],
 ["H:/OED/Collects/main/pdd/../parts/00000046", 2876070, 64003],
 ["H:/OED/Collects/main/pdd/../parts/00000045", 2811949, 64121],
 ["H:/OED/Collects/main/pdd/../parts/00000044", 2747938, 64011],
 ["H:/OED/Collects/main/pdd/../parts/00000043", 2682492, 65446],
 ["H:/OED/Collects/main/pdd/../parts/00000042", 2618478, 64014],
 ["H:/OED/Collects/main/pdd/../parts/00000041", 2554478, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000040", 2490472, 64006],
 ["H:/OED/Collects/main/pdd/../parts/00000039", 2426469, 64003],
 ["H:/OED/Collects/main/pdd/../parts/00000038", 2362468, 64001],
 ["H:/OED/Collects/main/pdd/../parts/00000037", 2298467, 64001],
 ["H:/OED/Collects/main/pdd/../parts/00000036", 2234464, 64003],
 ["H:/OED/Collects/main/pdd/../parts/00000035", 2170452, 64012],
 ["H:/OED/Collects/main/pdd/../parts/00000034", 2106433, 64019],
 ["H:/OED/Collects/main/pdd/../parts/00000033", 2042411, 64022],
 ["H:/OED/Collects/main/pdd/../parts/00000032", 1978223, 64188],
 ["H:/OED/Collects/main/pdd/../parts/00000031", 1914223, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000030", 1850165, 64058],
 ["H:/OED/Collects/main/pdd/../parts/00000029", 1786166, 63999],
 ["H:/OED/Collects/main/pdd/../parts/00000028", 1722161, 64005],
 ["H:/OED/Collects/main/pdd/../parts/00000027", 1658161, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000026", 1594119, 64042],
 ["H:/OED/Collects/main/pdd/../parts/00000025", 1530115, 64004],
 ["H:/OED/Collects/main/pdd/../parts/00000024", 1466116, 63999],
 ["H:/OED/Collects/main/pdd/../parts/00000023", 1402116, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000022", 1338069, 64047],
 ["H:/OED/Collects/main/pdd/../parts/00000021", 1274007, 64062],
 ["H:/OED/Collects/main/pdd/../parts/00000020", 1209952, 64055],
 ["H:/OED/Collects/main/pdd/../parts/00000019", 1145946, 64006],
 ["H:/OED/Collects/main/pdd/../parts/00000018", 1081703, 64243],
 ["H:/OED/Collects/main/pdd/../parts/00000017", 1017501, 64202],
 ["H:/OED/Collects/main/pdd/../parts/00000016", 953448, 64053],
 ["H:/OED/Collects/main/pdd/../parts/00000015", 889446, 64002],
 ["H:/OED/Collects/main/pdd/../parts/00000014", 825446, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000013", 761198, 64248],
 ["H:/OED/Collects/main/pdd/../parts/00000012", 697198, 64000],
 ["H:/OED/Collects/main/pdd/../parts/00000011", 633094, 64104],
 ["H:/OED/Collects/main/pdd/../parts/00000010", 569091, 64003],
 ["H:/OED/Collects/main/pdd/../parts/00000009", 504936, 64155],
 ["H:/OED/Collects/main/pdd/../parts/00000008", 440592, 64344],
 ["H:/OED/Collects/main/pdd/../parts/00000007", 375524, 65068],
 ["H:/OED/Collects/main/pdd/../parts/00000006", 311525, 63999],
 ["H:/OED/Collects/main/pdd/../parts/00000005", 247522, 64003],
 ["H:/OED/Collects/main/pdd/../parts/00000004", 183201, 64321],
 ["H:/OED/Collects/main/pdd/../parts/00000003", 119196, 64005],
 ["H:/OED/Collects/main/pdd/../parts/00000002", 54630, 64566],
 ["H:/OED/Collects/main/pdd/../parts/00000001", 1, 54629]]
